# Instala e carrega o pacote Benchmarking
if(!require(Benchmarking)) install.packages("Benchmarking"); require(Benchmarking)



# Carrega a base do Justia em Nmeros do site do CNJ.
temp <- tempfile()
download.file("https://www.cnj.jus.br/download/4431/justica-em-numeros/196206/02-set-2022.zip",temp)
jn <- read.csv2(unzip(temp,"JN_02-Set-2022.csv"), dec=",", header = TRUE, stringsAsFactor=F,na.strings="nd")
jn_secao <- read.csv2(unzip(temp,"JF_Secao_02_Set_2022.csv"),stringsAsFactor=F,na.strings="nd")
unlink(temp)

ano <- 2021

# Retira os totais da base.
colnames(jn)<-tolower(colnames(jn))
jn_estadual <- jn[!jn$seq_orgao %in% c(999991,999999) & jn$justica == "Estadual",]
jn_trabalho <- jn[!jn$seq_orgao %in% c(999991,999999) & jn$justica == "Trabalho",]
jn_federal<-jn_secao[jn_secao$secao != "total",]
colnames(jn_federal)<-tolower(colnames(jn_federal))


# Clculo do DEA.
jn_estadual$eff<-NA
jn_estadual$target.tbaix<-NA
dea_je<-NA
jn_trabalho$eff<-NA
jn_trabalho$target.tbaix<-NA
dea_jt<-NA

dea_estadual <- function(ano,rts="crs"){
                          base <- jn_estadual[jn_estadual$ano %in% ano,]
                          dea_je <<- dea(base[,c("cp.sus.fisc.penal","mag","dpjio","serv")],
                                         base[,"tbaix.sus.fisc.penal"], RTS=rts, ORIENTATION="out",SLACK=T)
                          jn_estadual[jn_estadual$ano %in% ano,"eff"] <<- 1/dea_je$eff
                          jn_estadual[jn_estadual$ano %in% ano,"target.tbaix"] <<- dea_je$eff*base$tbaix+dea_je$sy[,1]
                        }

dea_trabalho <- function(ano,rts="crs"){
  base <- jn_trabalho[jn_trabalho$ano %in% ano,]
  dea_jt <<- dea(base[,c("cp.sus.fisc.penal","mag","dpjio","serv")],
                 base[,"tbaix.sus.fisc.penal"], RTS=rts, ORIENTATION="out",SLACK=T)
  jn_trabalho[jn_trabalho$ano %in% ano,"eff"] <<- 1/dea_jt$eff
  jn_trabalho[jn_trabalho$ano %in% ano,"target.tbaix"] <<- dea_jt$eff*base$tbaix+dea_jt$sy[,1]
}


for (a in 2009:ano){
  dea_estadual(a)
  dea_trabalho(a)
}

DEA_JE<-jn_estadual[,c("ano","justica","sigla","dsc_tribunal","cp.sus.fisc.penal","mag","serv","dpjio","tbaix.sus.fisc.penal","eff","target.tbaix")]
DEA_JT<-jn_trabalho[,c("ano","justica","sigla","dsc_tribunal","cp.sus.fisc.penal","mag","serv","dpjio","tbaix.sus.fisc.penal","eff","target.tbaix")]



# Calcula IPC-Jus da Justia Federal
jn_federal <- jn_federal[jn_federal$secao != "TRU", c("chave","ano","sigla","dsc_tribunal","secao","dpjio","cp.sus.fisc.penal","tbaix.sus.fisc.penal","mag","serv")]
jn_federal$dpjio <- as.numeric(gsub(",",".",jn_federal$dpjio))

# Total 1 Grau por tribunal
tot1.1 <-aggregate(jn_federal[!jn_federal$secao %in% c("2 Grau","total"),6:ncol(jn_federal)],list(jn_federal$ano[!jn_federal$secao %in% c("2 Grau","total")],
                                                                                                   jn_federal$sigla[!jn_federal$secao %in% c("2 Grau","total")]),"sum")
colnames(tot1.1)[1]<- "ano"
colnames(tot1.1)[2]<- "sigla"
tot1.1$chave <- paste(tot1.1$sigla, "_1 Grau_",tot1.1$ano,sep="")
tot1.1$dsc_tribunal <- merge(data.frame(sigla=tot1.1$sigla),unique(jn_federal[,c("sigla","dsc_tribunal")]),all.x=T, sort=F)$dsc_tribunal
tot1.1$secao <- "1 Grau"
tot1.1 <- tot1.1[,colnames(jn_federal)]

# Total 1 Grau
tot1 <-aggregate(jn_federal[!jn_federal$secao %in% c("1 Grau","2 Grau","total"),6:ncol(jn_federal)],list(jn_federal$ano[!jn_federal$secao %in% c("1 Grau","2 Grau","total")]),"sum")
colnames(tot1)[1]<- "ano"
tot1$chave <- paste("TRF_1 Grau_",tot1$ano,sep="")
tot1$sigla <- "TRF"
tot1$dsc_tribunal <- "TRF"
tot1$secao <- "1 Grau"
tot1 <- tot1[,colnames(jn_federal)]

# Total 2 Grau
tot2 <-aggregate(jn_federal[jn_federal$secao %in% c("2 Grau"),6:ncol(jn_federal)],list(jn_federal$ano[jn_federal$secao %in% c("2 Grau")]),"sum")
colnames(tot2)[1]<- "ano"
tot2$chave <- paste("TRF_2 Grau_",tot2$ano,sep="")
tot2$sigla <- "TRF"
tot2$dsc_tribunal <- "TRF"
tot2$secao <- "2 Grau"
tot2 <- tot2[,colnames(jn_federal)]
jn_federal <- rbind(jn_federal,tot1.1, tot1, tot2)


# Calcula os indicadores utilizados no DEA
jn_federal$tcl <- ifelse(jn_federal$ano < 2015, 1-jn_federal$tbaix / jn_federal$tram, (jn_federal$cp-jn_federal$sus)/(jn_federal$tram-jn_federal$sus))
jn_federal$ips <- jn_federal$tbaix / jn_federal$serv
jn_federal$ipm <- jn_federal$tbaix / jn_federal$mag
jn_federal$dpjio_tbaix <- jn_federal$dpjio / jn_federal$tbaix

# Funo para o clculo do DEA
dea_federal <- function(base, input = c("cp.sus.fisc.penal","mag","dpjio","serv"), output = "tbaix.sus.fisc.penal", ano, rts="crs"){
  bd <- base[complete.cases(base[, c(input,output)]) & base$ano %in% ano & !base$secao %in% c("1 Grau", "total") & !base$sigla %in% c("TRF"),]
  deabd <- dea(as.matrix(bd[,input]), as.matrix(bd[,output]), RTS=rts, ORIENTATION="out",SLACK=T)
  bd$eff <- 1/deabd$eff
  bd$target.tbaix <- deabd$eff*bd[,output]+deabd$sy[,1]
  just <- "TRF"  
  trib <-aggregate(bd[,c(output,"target.tbaix")],list(bd$sigla),"sum")
  trib<-data.frame(paste(trib[,1],"_total_",ano, sep=""), trib[,output]/trib$target.tbaix, trib$target.tbaix)
  colnames(trib) <- c("chave","eff","target.tbaix")
  trib1 <-aggregate(bd[!bd$secao %in% c("2 Grau", "TRF"),c(output,"target.tbaix")],list(bd$sigla[!bd$secao %in% c("2 Grau", "TRF")]),"sum")
  trib1<-data.frame(paste(trib1[,1],"_1 Grau_",ano, sep=""), trib1[,output]/trib1$target.tbaix, trib1$target.tbaix)
  colnames(trib1) <- c("chave","eff","target.tbaix")
  tot<-data.frame(paste(just,"_total_",ano, sep=""), sum(bd[,output])/sum(bd$target.tbaix), sum(bd$target.tbaix))
  tot1<-data.frame(paste(just,"_1 Grau_",ano, sep=""), sum(bd[!bd$secao %in% c("2 Grau", "TRF"),output])/sum(bd$target.tbaix[!bd$secao %in% c("2 Grau", "TRF")]), 
                   sum(bd$target.tbaix[!bd$secao %in% c("2 Grau", "TRF")]))
  tot2<-data.frame(paste(just,"_2 Grau_",ano, sep=""), sum(bd[bd$secao %in% c("2 Grau"),output])/sum(bd$target.tbaix[bd$secao %in% c("2 Grau")]), 
                   sum(bd$target.tbaix[bd$secao %in% c("2 Grau")]))
  colnames(tot) <- c("chave","eff","target.tbaix")
  colnames(tot1) <- c("chave","eff","target.tbaix")
  colnames(tot2) <- c("chave","eff","target.tbaix")
  
  tot <- rbind(trib1,tot1,tot2,trib,tot)
  bd<-rbind(bd[,c("chave","eff","target.tbaix")],tot)
  return(bd)
}

# Calcula o IPCJus
dea.jf <- NULL
for (a in 2012:ano){
  dea.jf<-rbind(dea.jf,dea_federal(jn_federal, c("cp.sus.fisc.penal","mag","dpjio","serv"), "tbaix.sus.fisc.penal", ano = a))
}

DEA_JF<-merge(jn_federal, dea.jf, "chave",sort=F)


rm(jn,jn_secao,jn_estadual,jn_federal,jn_trabalho,tot1,tot1.1,tot2,
   a,dea_je,dea_jt,temp,dea_estadual,dea_federal,dea_trabalho,dea.jf)

# Fim
